package jp.co.riiiver.sdk.sample

import android.app.Activity
import android.content.Context
import android.graphics.Bitmap
import android.view.View
import android.webkit.CookieManager
import android.webkit.WebView
import com.riiiver.auth.*
import com.riiiver.sdk.ersdk.*
import jp.co.riiiver.sdk.sample.RiiiverSDKConfig.getDeviceIdArray
import jp.co.riiiver.sdk.sample.RiiiverSDKConfig.getDeviceIdList
import jp.co.riiiver.sdk.sample.RiiiverSDKConfig.pieceCorePackageName
import jp.co.riiiver.sdk.sample.RiiiverSDKConfig.vendorId
import jp.co.riiiver.sdk.sample.auth.ERAuthDelegateImpl
import timber.log.Timber
import java.util.*

object RiiiverSDKManager : ERSDKDelegate {

    private var erSdk: ERSDK? = null

    var accessToken: String? = null
    var idToken: String? = null
    private var context: Context? = null

    /**
     *  RiiiverSDKを初期化する
     */
    fun initialize(activity: Activity, complete: (isLoggedIn: Boolean) -> Unit) {
        context = activity
        ERAuth.initialize(activity, ERAuthDelegateImpl(activity)) {
            Timber.i("initialized")

            erSdk = ERSDK.Builder.with(activity)
                .includeBlockExecutorPackageName(pieceCorePackageName).build()

            // iiidea実行状態の監視を開始する
            erSdk!!.setDelegate(this)

            complete(ERAuth.getInstance().isLoggedIn)
        }
        // Preference関連の初期化
        IiideaSettingFragment.initialize(activity)
    }

    /**
     *  RiiiverSDKからWeb認証画面表示する
     *  set a web view to be  invoked by Riiiver SDK for the web page authentication.
     */
    fun setWebView(activity: Activity, webView: WebView, delegate: IERAuthWebViewDelegate) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            return
        }

        // ブラウザデバッグを行うための設定
//        WebView.setWebContentsDebuggingEnabled(true)

        webView.setLayerType(View.LAYER_TYPE_HARDWARE, null)

        // Display the authentication web page.  -  Web認証画面表示
        val deviceIds = getDeviceIdList()
        ERAuth.getInstance().openLogin(activity, webView, "", deviceIds, delegate)
    }

    /**
     *  RiiiverSDKから認証トークンを取得して保持する
     *  Get the auth token and cache it.
     *  @isSuccess true > Success to get the authentication info.  false > Failed to get the auth info.  認証情報の取得が成功した場合はtrue、失敗した場合はfalse
     *  @error the error information from Riiiver SDK.  -   RiiiverSDKから返却されたerror情報
     */
    fun getAWSToken(completion: (isSuccess: Boolean, error: Exception?) -> Unit) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            completion(false, null)
            return
        }

        if (!ERAuth.getInstance().isLoggedIn) {
            //未認証の場合、認証トークンを取得できないので終了 - Nothing to do... just return.
            Timber.e("updateAWSToken failed. Unauthenticated")
            completion(false, null)
            return
        }

        // RiiiverSDKから認証トークンを取得  -  Get the Auth token.
        ERAuth.getInstance().getToken(object : IERAuthGetTokenCallback {
            override fun onSuccess(token: ERAuthToken?) {
                Timber.i("getToken onSuccess.")

                if (token == null) {
                    Timber.e("getAWSToken failed. No token.")
                    completion(false, null)
                    return
                }

                if (token.accessToken == null) {
                    Timber.e("getAWSToken failed. No access-token.")
                    completion(false, null)
                    return
                }

                if (token.idToken == null) {
                    Timber.e("getAWSToken failed. No id-token.")
                    completion(false, null)
                    return
                }

                accessToken = token.accessToken
                idToken = token.idToken
                completion(true, null)
            }

            override fun onError(error: Exception?) {
                Timber.e("getToken onError.")
                completion(false, error)
            }
        })
    }

    /**
     * Pieceで使用しているAuth情報を削除する
     */
    fun deleteClientAuth(blockId: String, completion: (error: ERError?) -> Unit) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            completion(null)
            return
        }
        erSdk.deleteClientAuth(blockId) { error ->
            completion(error)
        }
    }

    /**
     * RiiiverSDKのセッション終了してログアウトする
     * End the session of the Riiiver authentication.  (Logout from Riiiver)
     */
    fun logout(completion: (isSuccess: Boolean, error: Exception?) -> Unit) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            completion(false, null)
            return
        }
        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        getAWSToken { isSuccess: Boolean, error: Exception? ->
            if (!isSuccess) {
                Timber.e(
                    "updateAWSToken failed. reason: %s",
                    if (error != null) error.message else ""
                )
                completion(false, null)
                return@getAWSToken
            }
            ERAuth.getInstance().logout(object : IERAuthCallback {
                override fun onSuccess() {
                    Timber.i("logout succeeded.")
                    completion(true, null)
                }

                override fun onCancel() {
                    Timber.e("onCancel in")
                }

                override fun onError(error: java.lang.Exception?) {
                    Timber.e("logout onError. %s", error.toString())
                    completion(false, error)
                }
            })
        }

    }


    /**
     *  RiiiverSDKからiiideaStore画面表示する
     *  Display Riiiver Store page via Riiiver SDK.
     */
    fun openStore(activity: Activity, webView: WebView, delegate: IERAuthWebViewDelegate) {
        // SDKの問題で一時的にcookieを削除する必要がある(SDKが対処された後は削除予定)
        val cookieManager: CookieManager = CookieManager.getInstance()
        cookieManager.removeAllCookies(null)

        webView.setLayerType(View.LAYER_TYPE_HARDWARE, null)
        val deviceIds = getDeviceIdList()
        ERAuth.getInstance().openStore(activity, webView, vendorId, deviceIds, delegate)
    }

    /**
     * RiiiverSDKから My iiidea 一覧を取得する
     * Get the my iiidea list from RiiiverSDK.
     *  @iiideaList  My iiidea 一覧. 取得に失敗した場合はnull
     *  @error the error information from Riiiver SDK.  -   RiiiverSDKから返却されたerror情報
     */
    fun getMyApplets(completion: (iiideaList: MutableList<ERApplet>?, error: Exception?) -> Unit) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            completion(null, null)
            return
        }

        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        getAWSToken { isSuccess: Boolean, error: Exception? ->
            if (!isSuccess) {
                Timber.e(
                    "updateAWSToken failed. reason: %s",
                    if (error != null) error.message else ""
                )
                completion(null, error)
                return@getAWSToken
            }

            erSdk.getMyApplets(getDeviceIdArray()) { appletList: MutableList<ERApplet>, erError: ERError? ->
                // Note: エラーがあっても取得できた My iiidea は返却されるため処理を継続することは可能 - Here, we can get the iiidea even if we get some errors.
                if (erError != null) {
                    Timber.e("getMyApplets failed. reason: %s", erError.message)
                }
                completion(appletList, null)
            }
        }
    }

    /**
     * 指定したiiideaをアクティベートする
     * Activate an terget iiidea
     *  @iiidea  アクティベートするiiidea
     *  @isSuccess アクティベートが成功した場合はtrue、失敗した場合はfalse
     *  @error RiiiverSDKから返却されたerror情報
     */
    fun activate(iiidea: ERApplet, completion: (isSuccess: Boolean, error: ERError?) -> Unit) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            completion(false, null)
            return
        }

        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        getAWSToken { isSuccess: Boolean, error: Exception? ->
            if (!isSuccess) {
                Timber.e(
                    "updateAWSToken failed. reason: %s",
                    if (error != null) error.message else ""
                )
                completion(false, null)
                return@getAWSToken
            }

            iiidea.activate(context) { _: Array<String>?, _: Array<ERServiceProxyData>?, erError: ERError? ->
                if (erError == null) {
                    Timber.d("activate succeeded.")
                    completion(true, null)
                } else {
                    completion(false, erError)
                }
            }
        }
    }

    /**
     * 指定したiiideaをディアクティベートする
     * Deactivate the target iiidea.
     *  @iiidea  ディアクティベートするiiidea  - target iiidea to deactivate.
     *  @isSuccess ディアクティベートが成功した場合はtrue、失敗した場合はfalse
     *  @error RiiiverSDKから返却されたerror情報
     */
    fun deactivate(iiidea: ERApplet, completion: (isSuccess: Boolean, error: Exception?) -> Unit) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            completion(false, null)
            return
        }

        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        getAWSToken { isSuccess: Boolean, error: Exception? ->
            if (!isSuccess) {
                Timber.e(
                    "updateAWSToken failed. reason: %s",
                    if (error != null) error.message else ""
                )
                completion(false, error)
                return@getAWSToken
            }
            iiidea.deactivate()
            Timber.d("deactivate succeeded.")
            completion(true, null)
        }
    }

    /**
     * 指定したiiideaを削除する
     * Remove the target iiidea.
     *  @iiidea  削除するiiidea  - target iiidea to remove.
     *  @isSuccess 削除が成功した場合はtrue、失敗した場合はfalse
     *  @error RiiiverSDKから返却されたerror情報
     */
    fun removeMyIiidea(
        iiidea: ERApplet,
        completion: (isSuccess: Boolean, error: ERError?) -> Unit
    ) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            completion(false, null)
            return
        }

        // セッションが切れないように毎回トークンを更新  -  Update the token to keep the auth session.
        getAWSToken { isSuccess: Boolean, error: Exception? ->
            if (!isSuccess) {
                Timber.e(
                    "updateAWSToken failed. reason: %s",
                    if (error != null) error.message else ""
                )
                completion(false, null)
                return@getAWSToken
            }
            erSdk.removeMyApplet(iiidea) { erError: ERError? ->
                if (erError == null) {
                    completion(true, null)
                } else {
                    completion(false, erError)
                }
            }
        }
    }

    /**
     * SDKが生成したUUIDを取得する
     * Get the UUID.
     */
    fun getUuid(): UUID? {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            return null
        }
        return erSdk.uuid
    }

    /**
     * 指定したURLの画像を取得する
     * Get the Image.
     * Deactivate the target iiidea.
     *  @imageUrl  取得する画像のURL
     *  @image 画像情報
     *  @s 画像情報
     *  @error RiiiverSDKから返却されたerror情報
     */
    fun getImage(
        imageUrl: String, completion: (image: Bitmap?, s: String?, error: ERError?) -> Unit
    ) {
        val erSdk = erSdk
        if (erSdk == null) {
            // initialize 未完了
            Timber.e("not initialized")
            completion(null, null, null)
            return
        }

        erSdk.getImage(imageUrl) { bitmap: Bitmap?, s: String?, erError: ERError? ->
            completion(bitmap, s, erError)
        }

    }

    /**
     * iiideaが実行開始した時に通知する
     * @param applet 起動したiiideaのインスタンス
     * @param error 起動に失敗した時のエラー
     */
    override fun startApplet(applet: ERApplet?, error: ERError?) {
        Timber.i("startApplet: error=${error}")
        if (applet == null) {
            Timber.e("startApplet: no iiidea")
            return
        }
        if (error != null) {
            Timber.e("startApplet: error %s", error.message)
            return
        }
        Timber.i("startApplet: appletId=${applet.id}, appletTitle=${applet.applet.title.get()}")
    }

    /**
     * iiideaが実行終了した時に通知する
     * @param applet 終了したiiideaのインスタンス
     * @param error iiideaが正しく終了しなかった時のエラー
     * @param reachBlock iiideaが終了した時の最終到達Block
     * @param output iiideaが正しく終了した際の、最後に到達したPieceが出力したデータ
     */
    override fun endApplet(
        applet: ERApplet?,
        error: ERError?,
        reachBlock: ReachBlock?,
        output: MutableMap<String, Any>?
    ) {
        Timber.i("endApplet: error=${error}, reachBlock=${reachBlock} output=${output}")
        if (applet == null) {
            Timber.e("endApplet: no iiidea")
            return
        }
        if (error != null) {
            Timber.e("endApplet: error %s", error.message)
            return
        }
        Timber.i("endApplet: appletId=${applet.id}, appletTitle=${applet.applet.title.get()}")
    }
}
