package jp.co.riiiver.sdk.sample.auth

import android.app.Activity
import android.content.Context
import android.content.Intent
import com.facebook.*
import com.facebook.AccessToken.AccessTokenRefreshCallback
import com.facebook.login.LoginBehavior
import com.facebook.login.LoginManager
import com.facebook.login.LoginResult
import com.google.android.gms.auth.api.signin.*
import com.google.android.gms.common.api.ApiException
import com.google.android.gms.common.util.IOUtils.*
import com.google.android.gms.tasks.Task
import com.riiiver.auth.*
import com.riiiver.auth.internal.Configuration
import org.json.JSONObject
import timber.log.Timber
import java.util.*


class ERAuthDelegateImpl(private val mContext: Context) : IERAuthDelegate {
    private var facebookCallbackManager: CallbackManager? = null
    private var googleSignInClient: GoogleSignInClient? = null
        get() {
            if (field == null) {
                field = try {
                    val clientId: String = Configuration.getJSONObject("GoogleSignIn").optString("ClientId")
                    val gso = GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                        .requestEmail()
                        .requestProfile()
                        .requestIdToken(clientId)
                        .build()
                    GoogleSignIn.getClient(mContext, gso)
                } catch (e: Exception) {
                    Timber.e(e.toString())
                    throw RuntimeException(e)
                }
            }
            return field
        }
    private var googleSignInCallback: IERAuthGoogleCallback? = null

    override fun refreshFacebook(callback: IERAuthFacebookCallback) {
        if (AccessToken.getCurrentAccessToken() != null) {
            AccessToken.refreshCurrentAccessTokenAsync(object : AccessTokenRefreshCallback {
                override fun OnTokenRefreshed(accessToken: AccessToken) {
                    callback.onSuccess(accessToken.token)
                }

                override fun OnTokenRefreshFailed(exception: FacebookException) {
                    AccessToken.setCurrentAccessToken(null)
                    callback.onFailed(exception)
                }
            })
            return
        } else {
            callback.onNotLoggedIn()
        }
    }

    override fun loginFacebook(activity: Activity?, callback: IERAuthFacebookCallback) {
        facebookCallbackManager = CallbackManager.Factory.create()
        LoginManager.getInstance().loginBehavior = LoginBehavior.WEB_VIEW_ONLY
        LoginManager.getInstance().registerCallback(facebookCallbackManager, object : FacebookCallback<LoginResult> {
            override fun onSuccess(loginResult: LoginResult) {
                if (loginResult.accessToken != null) {
                    callback.onSuccess(loginResult.accessToken.token)
                } else {
                    callback.onCancelled()
                }
            }

            override fun onCancel() {
                callback.onCancelled()
            }

            override fun onError(error: FacebookException) {
                callback.onFailed(error)
            }
        })
        try {
            val perms: String = Configuration.getJSONObject("FacebookSignIn").optString("Permissions")
            val permissions = listOf(*perms.split(",").toTypedArray())
            LoginManager.getInstance().logInWithReadPermissions(activity, permissions)
        } catch (e: Exception) {
            Timber.e(e.toString())
        }
    }

    override fun logoutFacebook() {
        if (AccessToken.getCurrentAccessToken() != null) {
            GraphRequest(AccessToken.getCurrentAccessToken(), "/me/permissions/", null, HttpMethod.DELETE, GraphRequest.Callback { LoginManager.getInstance().logOut() }).executeAsync()
        }
    }

    override fun refreshGoogle(callback: IERAuthGoogleCallback?) {
        googleSignInCallback = callback
        val task = googleSignInClient!!.silentSignIn()
        task.continueWith<Any> { result ->
            handleGoogleSignInResult(result)
            null
        }
    }

    override fun loginGoogle(activity: Activity, callback: IERAuthGoogleCallback?) {
        googleSignInCallback = callback
        val signInIntent = googleSignInClient!!.signInIntent
        activity.startActivityForResult(signInIntent, RC_SIGN_IN)
    }

    /**
     * Googleのログインの完了処理を行う。
     * Googleのアクセストークンを使って、サーバで認証する。
     *
     */
    private fun handleGoogleSignInResult(completedTask: Task<GoogleSignInAccount>) {
        try {
            val account = completedTask.getResult(ApiException::class.java)
            val params: MutableMap<String, String?> = HashMap()
            params["name"] = account!!.displayName
            params["photo"] = account.photoUrl.toString()
            params["token"] = account.idToken
            googleSignInCallback!!.onSuccess(params)
        } catch (e: ApiException) {
            if (e.statusCode == GoogleSignInStatusCodes.SIGN_IN_CANCELLED) {
                googleSignInCallback!!.onCancelled()
            } else {
                googleSignInCallback!!.onFailed(e)
            }
        } finally {
            googleSignInCallback = null
        }
    }

    override fun logoutGoogle() {
        googleSignInClient!!.signOut()
    }

    private var isLoadedAppleCredentials = false
    private var appleAccessToken: String? = null
    private var appleRefreshToken: String? = null
    private var appleIdToken: String? = null
    private var appleExpiresAt: Date? = null
    private fun loadPrivateKey(keyFile: String): ByteArray? {
        try {
            mContext.assets.open("$keyFile.p8").use { `is` -> return readInputStreamFully(`is`) }
        } catch (e: Exception) {
            Timber.e(e.toString())
        }
        return null
    }

    override fun checkSignInWithApple(callback: IERAuthAppleCallback) {
        try {
            if (!isLoadedAppleCredentials) {
                val json: JSONObject = ERAuth.getInstance().readSignInWithAppleParams()
                appleAccessToken = json.getString("accessToken")
                appleRefreshToken = json.getString("refreshToken")
                appleIdToken = json.getString("idToken")
                val epoch = json.getLong("expiresAt")
                val cal = Calendar.getInstance()
                cal.timeInMillis = epoch * 1000L
                appleExpiresAt = cal.time
                isLoadedAppleCredentials = true
            }
            val conf: JSONObject = ERAuth.getInstance().getConfig("SignInWithApple")
            val teamId = conf.optString("teamId")
            val keyId = conf.optString("keyId")
            val keyFile = conf.optString("privateKey")
            val serviceId = conf.optString("serviceId")
            val privateKey = loadPrivateKey(keyFile)
            val secret: String = ERAuth.appleClientSecret(teamId, serviceId, keyId, privateKey)
            callback.with(serviceId, secret, appleIdToken, appleRefreshToken)
        } catch (e: Exception) {
            callback.with(e)
        }
    }

    override fun clearAppleTokens() {
        appleAccessToken = null
        appleRefreshToken = null
        appleIdToken = null
        appleExpiresAt = null
        ERAuth.getInstance().writeSignInWithAppleParams(null)
    }

    override fun updateAppleTokens(accessToken: String?, refreshToken: String?, idToken: String?, expiresAt: Date?) {
        appleAccessToken = accessToken
        appleExpiresAt = expiresAt
        if (refreshToken != null) {
            appleRefreshToken = refreshToken
        }
        if (idToken != null) {
            appleIdToken = idToken
        }
        val json = JSONObject()
        try {
            if (appleAccessToken != null) {
                json.put("accessToken", appleAccessToken)
            }
            if (appleExpiresAt != null) {
                val cal = Calendar.getInstance()
                cal.time = appleExpiresAt
                json.put("expiresAt", cal.timeInMillis / 1000)
            }
            if (appleIdToken != null) {
                json.put("idToken", appleIdToken)
            }
            if (appleRefreshToken != null) {
                json.put("refreshToken", appleRefreshToken)
            }
            ERAuth.getInstance().writeSignInWithAppleParams(json)
        } catch (e: Exception) {
            Timber.e(e, "updateAppleTokens")
        }
    }

    override fun loginVendor(activity: Activity?, callback: IERAuthVendorCallback) {
        val result: MutableMap<String, String> = HashMap()
        result["region"] = "JP"
        callback.onSuccess(result)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        if (FacebookSdk.isFacebookRequestCode(requestCode)) {
            facebookCallbackManager!!.onActivityResult(requestCode, resultCode, data)
            return true
        }
        if (requestCode == RC_SIGN_IN) {
            val task = GoogleSignIn.getSignedInAccountFromIntent(data)
            handleGoogleSignInResult(task)
            return true
        }
        return false
    }

    companion object {
        private const val RC_SIGN_IN = 10001
    }

}
